 
bl_info = {
    "name": "AltView",
    "author": "Iurii Tiuzhin",
    "version": (1, 0),
    "blender": (4, 2, 5),
    "location": "View3D",
    "description": "Use the hotkey to switch between different views and lock/unlock rotation.",
    "category": "3D View",
}

import bpy
import math
from mathutils import Vector, Quaternion, Euler
from bpy.props import StringProperty

# Функция для преобразования градусов в радианы
def deg_to_rad(deg):
    return deg * math.pi / 180

# Список всех видов
ALL_VIEWS = {
    'СЗАДИ-СЛЕВА 1': (90, 0, 225),
    'СЗАДИ-СЛЕВА 2': (90, -45, 225),
    'СЗАДИ-СЛЕВА 3': (-45, -90, 0),
    'СЗАДИ-СЛЕВА 4': (-90, -45, 45),
    'СЗАДИ-СЛЕВА 5': (-90, 0, 45),
    'СЗАДИ-СЛЕВА 6': (-90, 45, 45),
    'СЗАДИ-СЛЕВА 7': (-135, 90, 0),
    'СЗАДИ-СЛЕВА 8': (-90, 135, 45),


    'СЛЕВА-СПЕРЕДИ 1': (90, 0, -45),
    'СЛЕВА-СПЕРЕДИ 2': (90, -45, -45),
    'СЛЕВА-СПЕРЕДИ 3': (45, -90, -0),
    'СЛЕВА-СПЕРЕДИ 4': (90, -135, -45),
    'СЛЕВА-СПЕРЕДИ 5': (90, -180, -45),
    'СЛЕВА-СПЕРЕДИ 6': (90, -225, -45),
    'СЛЕВА-СПЕРЕДИ 7': (135, -270, 0),
    'СЛЕВА-СПЕРЕДИ 8': (90, -315, -45),


    'СПЕРЕДИ-СПРАВА 1': (90, 0, 45),
    'СПЕРЕДИ-СПРАВА 2': (90, -45, 45),
    'СПЕРЕДИ-СПРАВА 3': (135, -90, 0),
    'СПЕРЕДИ-СПРАВА 4': (90, -135, 45),
    'СПЕРЕДИ-СПРАВА 5': (90, -180, 45),
    'СПЕРЕДИ-СПРАВА 6': (90, -225, 45),
    'СПЕРЕДИ-СПРАВА 7': (45, -275, 0),
    'СПЕРЕДИ-СПРАВА 8': (90, -315, 45),

    'СПРАВА-СЗАДИ 1': (90, 0, 135),
    'СПРАВА-СЗАДИ 2': (90, -45, 135),
    'СПРАВА-СЗАДИ 3': (225, -90, 0),
    'СПРАВА-СЗАДИ 4': (270, -45, -45),
    'СПРАВА-СЗАДИ 5': (270, 0, -45),
    'СПРАВА-СЗАДИ 6': (270, 45, -45),
    'СПРАВА-СЗАДИ 7': (315, 90, 0),
    'СПРАВА-СЗАДИ 8': (270, 135, -45),

    'СЗАДИ-СЛЕВА-СВЕРХУ 1': (45, 0, 225),
    'СЗАДИ-СЛЕВА-СВЕРХУ 2': (35.2644, -30, 260.264),
    'СЗАДИ-СЛЕВА-СВЕРХУ 3': (0, -45, 315),
    'СЗАДИ-СЛЕВА-СВЕРХУ 4': (-35.2644, -30, 9.736),
    'СЗАДИ-СЛЕВА-СВЕРХУ 5': (-45, 0, 45),
    'СЗАДИ-СЛЕВА-СВЕРХУ 6': (-35.2644, 30, 80.264),
    'СЗАДИ-СЛЕВА-СВЕРХУ 7': (0, 45, 135),
    'СЗАДИ-СЛЕВА-СВЕРХУ 8': (35.2644, 30, 189.736),

    'СЛЕВА-СПЕРЕДИ-СВЕРХУ 1': (45, 0, -45),
    'СЛЕВА-СПЕРЕДИ-СВЕРХУ 2': (35.2644, -30, -9.7356),
    'СЛЕВА-СПЕРЕДИ-СВЕРХУ 3': (0, -45, 45),
    'СЛЕВА-СПЕРЕДИ-СВЕРХУ 4': (-35.2644, -30, 99.7356),
    'СЛЕВА-СПЕРЕДИ-СВЕРХУ 5': (-45, 0, 135),
    'СЛЕВА-СПЕРЕДИ-СВЕРХУ 6': (-35.2644, 30, 170.264),
    'СЛЕВА-СПЕРЕДИ-СВЕРХУ 7': (0, 45, 225),
    'СЛЕВА-СПЕРЕДИ-СВЕРХУ 8': (35.2644, 30, 279.736),

    'СПРАВА-СЗАДИ-СВЕРХУ 1': (45, 0, 135),
    'СПРАВА-СЗАДИ-СВЕРХУ 2': (35.2644, -30, 170.264),
    'СПРАВА-СЗАДИ-СВЕРХУ 3': (0, -45, 225),
    'СПРАВА-СЗАДИ-СВЕРХУ 4': (-35.2644, -30, 279.736),
    'СПРАВА-СЗАДИ-СВЕРХУ 5': (-45, 0, 315),
    'СПРАВА-СЗАДИ-СВЕРХУ 6': (-35.2644, 30, 350.264),
    'СПРАВА-СЗАДИ-СВЕРХУ 7': (0, 45, 45),
    'СПРАВА-СЗАДИ-СВЕРХУ 8': (35.2644, 30, 99.736),

    'СЛЕВА-СВЕРХУ 1': (45, 0, -90),
    'СЛЕВА-СВЕРХУ 2': (35.2644, -30, -54.7356),
    'СЛЕВА-СВЕРХУ 3': (0, -45, 0),
    'СЛЕВА-СВЕРХУ 4': (-35.2644, -30, 54.7356),
    'СЛЕВА-СВЕРХУ 5': (-45, 0, 90),
    'СЛЕВА-СВЕРХУ 6': (-35.2644, 30, 125.264),
    'СЛЕВА-СВЕРХУ 7': (0, 45, 180),
    'СЛЕВА-СВЕРХУ 8': (35.2644, 30, 234.736),

    'СПЕРЕДИ-СВЕРХУ 1': (45, 0, 0),
    'СПЕРЕДИ-СВЕРХУ 2': (35.2644, -30, 35.2644),
    'СПЕРЕДИ-СВЕРХУ 3': (0, -45, 90),
    'СПЕРЕДИ-СВЕРХУ 4': (-35.2644, -30, 144.736),
    'СПЕРЕДИ-СВЕРХУ 5': (-45, 0, 180),
    'СПЕРЕДИ-СВЕРХУ 6': (-35.2644, 30, 215.264),
    'СПЕРЕДИ-СВЕРХУ 7': (0, 45, 270),
    'СПЕРЕДИ-СВЕРХУ 8': (35.2644, 30, 324.736),

    'СПЕРЕДИ-СПРАВА-СВЕРХУ 1': (45, 0, 45),
    'СПЕРЕДИ-СПРАВА-СВЕРХУ 2': (35.2644, -30, 80.2644),
    'СПЕРЕДИ-СПРАВА-СВЕРХУ 3': (0, -45, 135),
    'СПЕРЕДИ-СПРАВА-СВЕРХУ 4': (-35.2644, -30, 189.736),
    'СПЕРЕДИ-СПРАВА-СВЕРХУ 5': (-45, 0, 225),
    'СПЕРЕДИ-СПРАВА-СВЕРХУ 6': (-35.2644, 30, 260.264),
    'СПЕРЕДИ-СПРАВА-СВЕРХУ 7': (0, 45, 315),
    'СПЕРЕДИ-СПРАВА-СВЕРХУ 8': (35.2644, 30, 9.736),

    'СПРАВА-СВЕРХУ 1': (45, 0, 90),
    'СПРАВА-СВЕРХУ 2': (35.2644, -30, 125.264),
    'СПРАВА-СВЕРХУ 3': (0, -45, 180),
    'СПРАВА-СВЕРХУ 4': (-35.2644, -30, 234.736),
    'СПРАВА-СВЕРХУ 5': (-45, 0, 270),
    'СПРАВА-СВЕРХУ 6': (-35.2644, 30, 305.264),
    'СПРАВА-СВЕРХУ 7': (0, 45, 0),
    'СПРАВА-СВЕРХУ 8': (35.2644, 30, 54.736),

    'СЗАДИ-СЛЕВА-СНИЗУ 1': (135, 0, 225),
    'СЗАДИ-СЛЕВА-СНИЗУ 2': (144.736, -30, 189.736),
    'СЗАДИ-СЛЕВА-СНИЗУ 3': (180, -45, 135),
    'СЗАДИ-СЛЕВА-СНИЗУ 4': (215.264, -30, 80.2644),
    'СЗАДИ-СЛЕВА-СНИЗУ 5': (225, 0, 45),
    'СЗАДИ-СЛЕВА-СНИЗУ 6': (215.264, 30, 9.73566),
    'СЗАДИ-СЛЕВА-СНИЗУ 7': (180, 45, -45),
    'СЗАДИ-СЛЕВА-СНИЗУ 8': (144.736, 30, -99.7356),

    'СЗАДИ-СНИЗУ 1': (135, 0, 180),
    'СЗАДИ-СНИЗУ 2': (144.736, -30, 144.736),
    'СЗАДИ-СНИЗУ 3': (180, -45, 90),
    'СЗАДИ-СНИЗУ 4': (215.264, -30, 35.2644),
    'СЗАДИ-СНИЗУ 5': (225, 0, 0),
    'СЗАДИ-СНИЗУ 6': (215.264, 30, -35.2644),
    'СЗАДИ-СНИЗУ 7': (180, 45, -90),
    'СЗАДИ-СНИЗУ 8': (144.736, 30, -144.736),

    'СЛЕВА-СНИЗУ 1': (135, 0, -90),
    'СЛЕВА-СНИЗУ 2': (144.736, -30, -125.264),
    'СЛЕВА-СНИЗУ 3': (180, -45, -180),
    'СЛЕВА-СНИЗУ 4': (215.264, -30, -234.736),
    'СЛЕВА-СНИЗУ 5': (225, 0, -270),
    'СЛЕВА-СНИЗУ 6': (215.264, 30, -305.264),
    'СЛЕВА-СНИЗУ 7': (180, 45, 0),
    'СЛЕВА-СНИЗУ 8': (144.736, 30, -54.736),

    'СЛЕВА-СПЕРЕДИ-СНИЗУ 1': (135, 0, -45),
    'СЛЕВА-СПЕРЕДИ-СНИЗУ 2': (144.736, -30, -80.2644),
    'СЛЕВА-СПЕРЕДИ-СНИЗУ 3': (180, -45, -135),
    'СЛЕВА-СПЕРЕДИ-СНИЗУ 4': (215.264, -30, -189.736),
    'СЛЕВА-СПЕРЕДИ-СНИЗУ 5': (225, 0, -225),
    'СЛЕВА-СПЕРЕДИ-СНИЗУ 6': (215.264, 30, -260.264),
    'СЛЕВА-СПЕРЕДИ-СНИЗУ 7': (180, 45, -315),
    'СЛЕВА-СПЕРЕДИ-СНИЗУ 8': (144.736, 30, -9.736),

    'СПЕРЕДИ-СНИЗУ 1': (135, 0, 0),
    'СПЕРЕДИ-СНИЗУ 2': (144.736, -30, -35.2644),
    'СПЕРЕДИ-СНИЗУ 3': (180, -45, -90),
    'СПЕРЕДИ-СНИЗУ 4': (215.264, -30, -144.736),
    'СПЕРЕДИ-СНИЗУ 5': (225, 0, -180),
    'СПЕРЕДИ-СНИЗУ 6': (215.264, 30, -215.264),
    'СПЕРЕДИ-СНИЗУ 7': (180, 45, -270),
    'СПЕРЕДИ-СНИЗУ 8': (144.736, 30, -324.736),

    'СПЕРЕДИ-СПРАВА-СНИЗУ 1': (135, 0, 45),
    'СПЕРЕДИ-СПРАВА-СНИЗУ 2': (144.736, -30, 9.7356),
    'СПЕРЕДИ-СПРАВА-СНИЗУ 3': (180, -45, -45),
    'СПЕРЕДИ-СПРАВА-СНИЗУ 4': (215.264, -30, -99.7356),
    'СПЕРЕДИ-СПРАВА-СНИЗУ 5': (225, 0, -135),
    'СПЕРЕДИ-СПРАВА-СНИЗУ 6': (215.264, 30, -170.264),
    'СПЕРЕДИ-СПРАВА-СНИЗУ 7': (180, 45, -225),
    'СПЕРЕДИ-СПРАВА-СНИЗУ 8': (144.736, 30, -279.736),

    'СПРАВА-СЗАДИ-СНИЗУ 1': (135, 0, 135),
    'СПРАВА-СЗАДИ-СНИЗУ 2': (144.736, -30, 99.7356),
    'СПРАВА-СЗАДИ-СНИЗУ 3': (180, -45, 45),
    'СПРАВА-СЗАДИ-СНИЗУ 4': (215.264, -30, -9.7356),
    'СПРАВА-СЗАДИ-СНИЗУ 5': (225, 0, -45),
    'СПРАВА-СЗАДИ-СНИЗУ 6': (215.264, 30, -80.2644),
    'СПРАВА-СЗАДИ-СНИЗУ 7': (180, 45, -135),
    'СПРАВА-СЗАДИ-СНИЗУ 8': (144.736, 30, -189.736),

    'СПРАВА-СНИЗУ 1': (135, 0, 90),
    'СПРАВА-СНИЗУ 2': (144.736, -30, 54.7356),
    'СПРАВА-СНИЗУ 3': (180, -45, 0),
    'СПРАВА-СНИЗУ 4': (215.264, -30, -54.7356),
    'СПРАВА-СНИЗУ 5': (225, 0, -90),
    'СПРАВА-СНИЗУ 6': (215.264, 30, -125.264),
    'СПРАВА-СНИЗУ 7': (180, 45, -180),
    'СПРАВА-СНИЗУ 8': (144.736, 30, -234.736),

    'СЛЕВА 1': (90, 0, -90),
    'СЛЕВА 2': (90, -45, -90),
    'СЛЕВА 3': (0, -90, 0),
    'СЛЕВА 4': (90, -135, -90),
    'СЛЕВА 5': (90, -180, -90),
    'СЛЕВА 6': (90, -225, -90),
    'СЛЕВА 7': (180, -270, 0),
    'СЛЕВА 8': (90, -315, -90),

    'СВЕРХУ 1': (0, 0, 0),
    'СВЕРХУ 2': (0, 0, 45),
    'СВЕРХУ 3': (0, 0, 90),
    'СВЕРХУ 4': (0, 0, 135),
    'СВЕРХУ 5': (0, 0, 180),
    'СВЕРХУ 6': (0, 0, 225),
    'СВЕРХУ 7': (0, 0, 270),
    'СВЕРХУ 8': (0, 0, 315),

     'СЗАДИ 1': (90, 0, 180),
     'СЗАДИ 2': (90, -45, 180),
     'СЗАДИ 3': (-90, -90, 0),
     'СЗАДИ 4': (-90, -45, 0),
     'СЗАДИ 5': (-90, 0, 0),
     'СЗАДИ 6': (-90, 45, 0),
     'СЗАДИ 7': (-90, 90, 0),
     'СЗАДИ 8': (-90, 135, 0),

     'СНИЗУ 1': (180, 0, 0),
     'СНИЗУ 2': (180, 0, -45),
     'СНИЗУ 3': (180, 0, -90),
     'СНИЗУ 4': (180, 0, -135),
     'СНИЗУ 5': (180, 0, -180),
     'СНИЗУ 6': (180, 0, -225),
     'СНИЗУ 7': (180, 0, -270),
     'СНИЗУ 8': (180, 0, -315),

     'СПЕРЕДИ 1': (90, 0, 0),
     'СПЕРЕДИ 2': (90, -45, 0),
     'СПЕРЕДИ 3': (90, -90, 0),
     'СПЕРЕДИ 4': (90, -135, 0),
     'СПЕРЕДИ 5': (90, -180, 0),
     'СПЕРЕДИ 6': (90, -225, 0),
     'СПЕРЕДИ 7': (90, -270, 0),
     'СПЕРЕДИ 8': (90, -315, 0),

     'СПРАВА 1': (90, 0, 90),
     'СПРАВА 2': (90, -45, 90),
     'СПРАВА 3': (180, -90, 0),
     'СПРАВА 4': (270, -45, -90),
     'СПРАВА 5': (270, 0, -90),
     'СПРАВА 6': (270, 45, -90),
     'СПРАВА 7': (0, 90, 0),
     'СПРАВА 8': (90, 45, 90),

}

# Получаем кватернион из углов Эйлера
def get_quaternion_from_euler(euler_angles):
    rad_angles = [deg_to_rad(angle) for angle in euler_angles]
    return Euler(rad_angles, 'XYZ').to_quaternion()

# Вычисляем угол между двумя кватернионами
def get_angle_between_quaternions(q1, q2):
    dot_product = q1.dot(q2)
    return 2 * math.acos(abs(dot_product))

# Класс настроек аддона
class AltViewPreferences(bpy.types.AddonPreferences):
    bl_idname = __name__

    hotkey: StringProperty(
        name="Hotkey",
        description="Hotkey for switching between views and locking/unlocking rotation\n"
                    "(e.g., \"Spacebar\" for space or \"Ctrl+Spacebar\")",
        default="Shift+V",
        update=lambda self, context: update_hotkey(self, context)
    )

    def draw(self, context):
        layout = self.layout
        layout.label(text="To set a hotkey:")
        layout.label(text="1. Enter your desired combination in the Hotkey field.")
        layout.label(text="2. Press Save User Settings.")
        layout.separator()
        layout.prop(self, "hotkey")

# Парсим строку хоткея
def parse_hotkey(hotkey_str):
    parts = [part.strip() for part in hotkey_str.split('+')]
    modifiers = {
        'shift': False,
        'ctrl': False,
        'alt': False,
        'oskey': False,
    }
    if not parts:
        return {'type': 'NONE', **modifiers}

    key_part = parts[-1].upper()
    key_translate = {
        'SPACEBAR': 'SPACE',
        'SPACE': 'SPACE',
        'LEFTMOUSE': 'LEFTMOUSE',
        'RIGHTMOUSE': 'RIGHTMOUSE',
        'MIDDLEMOUSE': 'MIDDLEMOUSE',
    }
    key_type = key_translate.get(key_part, key_part)

    for part in parts[:-1]:
        lower_part = part.lower()
        if lower_part == 'shift':
            modifiers['shift'] = True
        elif lower_part == 'ctrl':
            modifiers['ctrl'] = True
        elif lower_part == 'alt':
            modifiers['alt'] = True
        elif lower_part in ('oskey', 'cmd'):
            modifiers['oskey'] = True

    return {'type': key_type, **modifiers}

# Обновление хоткеев
def update_hotkey(self, context):
    wm = context.window_manager
    kc = wm.keyconfigs.addon
    if not kc:
        return

    # Удаляем старые привязки клавиш
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()

    parsed = parse_hotkey(self.hotkey)

    km = kc.keymaps.get('3D View')
    if not km:
        km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')

    kmi = km.keymap_items.new(
        VIEW3D_OT_alt_view_switch.bl_idname,
        type=parsed['type'],
        value='PRESS',
        shift=parsed['shift'],
        ctrl=parsed['ctrl'],
        alt=parsed['alt'],
        oskey=parsed['oskey']
    )
    addon_keymaps.append((km, kmi))

# Оператор для переключения вида и блокировки вращения
class VIEW3D_OT_alt_view_switch(bpy.types.Operator):
    """Switches between view modes and locks/unlocks rotation"""
    bl_idname = "view3d.alt_view_switch"
    bl_label = "Switch Between Views and Lock/Unlock Rotation"
    bl_description = "Switches between view modes and locks/unlocks rotation based on state"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return context.area.type == 'VIEW_3D'  # Проверка, что мы находимся в 3D View

    def execute(self, context):
        active_area = context.area  # Текущая активная область
        space_data = active_area.spaces.active  # Данные области

        if space_data.region_3d.lock_rotation:  # Если вращение заблокировано
            space_data.region_3d.lock_rotation = False
            space_data.region_3d.view_perspective = 'PERSP'
        else:
            current_rotation = space_data.region_3d.view_rotation

            min_angle = float('inf')
            best_rotation = None

            for view_name, angles in ALL_VIEWS.items():
                view_rotation = get_quaternion_from_euler(angles)
                angle = get_angle_between_quaternions(current_rotation, view_rotation)

                if angle < min_angle:
                    min_angle = angle
                    best_rotation = view_rotation

            if best_rotation:
                space_data.region_3d.view_rotation = best_rotation
                space_data.region_3d.view_perspective = 'ORTHO'
                space_data.region_3d.lock_rotation = True

        return {'FINISHED'}

addon_keymaps = []  # Массив для хранения привязок клавиш

# Регистрация аддона
def register():
    bpy.utils.register_class(AltViewPreferences)
    bpy.utils.register_class(VIEW3D_OT_alt_view_switch)

    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        prefs = bpy.context.preferences.addons[__name__].preferences
        hotkey_str = prefs.hotkey if hasattr(prefs, 'hotkey') else "Shift+V"

        parsed = parse_hotkey(hotkey_str)

        km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
        kmi = km.keymap_items.new(
            VIEW3D_OT_alt_view_switch.bl_idname,
            type=parsed['type'],
            value='PRESS',
            shift=parsed['shift'],
            ctrl=parsed['ctrl'],
            alt=parsed['alt'],
            oskey=parsed['oskey']
        )
        addon_keymaps.append((km, kmi))

# Отмена регистрации аддона
def unregister():
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()

    bpy.utils.unregister_class(VIEW3D_OT_alt_view_switch)
    bpy.utils.unregister_class(AltViewPreferences)

if __name__ == "__main__":
    register()
